#############################################################################
## Exponentially weighted moving average.
##
## --------------------------------------------------------------------------
## Copyright (C) 2007 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## $Id: ewma.rb,v 1.4 2008/02/20 01:57:59 youngh Exp $
#############################################################################

module ArkUtil

class EWMA

  def initialize(num_samples)
    @alpha = 2.0 / (num_samples + 1)
    @beta = 1.0 - @alpha
    @average = nil
    @value = nil
  end

  def add(value)
    if @average
      @last_value, @value = @value, value
      @average = @alpha * @last_value + @beta * @average
    elsif @value
      @last_value, @value = @value, value      
      @average = @last_value
    else
      @value = value
      0
    end
  end

  def average
    @average || 0
  end

end

end  # module ArkUtil
