#############################################################################
## A special-purpose multiplicative linear congruential generator.
##
## The parameters have been carefully chosen to match the needs of "randomly"
## scanning every /24 in team-probing Ark measurements.
##
## --------------------------------------------------------------------------
## Copyright (C) 2007 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## $Id: mlcg.rb,v 1.7 2008/02/20 01:58:20 youngh Exp $
#############################################################################

module ArkUtil

class MLCG

  # From Pierre L'Ecuyer, "Tables of Linear Congruential Generators of
  # Different Sizes and Good Lattice Structure", AMS, 1999.
  A = 6423135
  M = 16777213 # == 2^24 - 3

  def initialize(seed=1)
    @value = seed
  end

  def seed=(seed)
    @value = seed
  end

  def seed
    @value
  end

  def next
    @value = (A * @value) % M
  end

end

#===========================================================================
if __FILE__ == $0
  require 'ostruct'
  require 'optparse'

  options = OpenStruct.new
  opts = OptionParser.new

  opts.on("-d", "--[no-]debug", TrueClass,  "show detailed progress") do |v|
    options.debug = v
  end

  opts.on("-n", "--number", "=NUM", "number of values to generate") do |v|
    options.number = v.to_i
  end

  opts.on("-o", "--offset", "=NUM", "offset from seed to begin generation") do |v|
    options.offset = v.to_i
  end

  opts.on("-s", "--seed", "=NUM", "seed value") do |v|
    options.seed = v.to_i
  end

  begin
    ARGV.replace opts.parse(*ARGV)
  rescue OptionParser::ParseError
    $stderr.puts "ERROR: " + $!
    puts opts
    exit 1
  end

  options.number ||= 10
  options.offset ||= 0
  options.seed ||= 1

  mlcg = MLCG.new options.seed

  options.offset.times { mlcg.next }

  options.number.times do
    puts mlcg.next
  end

end

end  # module ArkUtil
